@@ -82,14 +82,6 @@ class InvaliddUserException(Exception):
         self.message = msg
 
 
-class IncorrectVMCrecentialsException(Exception):
-    """ Invalid InfrastructureManager credentials """
-
-    def __init__(self, msg="Incorrect VM credentials"):
-        Exception.__init__(self, msg)
-        self.message = msg
-
-
 class DisabledFunctionException(Exception):
     """ Disabled function called"""
 
@@ -593,6 +585,10 @@ class InfrastructureManager:
         # We are going to start adding resources
         sel_inf.set_adding()
 
+        if sel_inf.deleted:
+            InfrastructureManager.logger.info("Inf ID: %s: Deleted Infrastructure. Stop deploying!" % sel_inf.id)
+            return []
+
         # Launch every group in the same cloud provider
         deployed_vm = {}
         for deploy_group in deploy_groups:
@@ -1330,7 +1326,8 @@ class InfrastructureManager:
         if Config.OIDC_SCOPES and Config.OIDC_CLIENT_ID and Config.OIDC_CLIENT_SECRET:
             success, res = OpenIDClient.get_token_introspection(token,
                                                                 Config.OIDC_CLIENT_ID,
-                                                                Config.OIDC_CLIENT_SECRET)
+                                                                Config.OIDC_CLIENT_SECRET,
+                                                                Config.VERIFI_SSL)
             if not success:
                 raise InvaliddUserException("Invalid InfrastructureManager credentials. "
                                             "Invalid token or Client credentials.")
@@ -1353,7 +1350,7 @@ class InfrastructureManager:
 
         try:
             # Now try to get user info
-            success, userinfo = OpenIDClient.get_user_info_request(token)
+            success, userinfo = OpenIDClient.get_user_info_request(token, Config.VERIFI_SSL)
             if success:
                 # convert to username to use it in the rest of the IM
                 im_auth['username'] = IM.InfrastructureInfo.InfrastructureInfo.OPENID_USER_PREFIX
@@ -1378,10 +1375,10 @@ class InfrastructureManager:
         im_auth = auth.getAuthInfo("InfrastructureManager")
 
         if not im_auth:
-            raise IncorrectVMCrecentialsException("No credentials provided for the InfrastructureManager.")
+            raise InvaliddUserException("No credentials provided for the InfrastructureManager.")
 
         if Config.FORCE_OIDC_AUTH and "token" not in im_auth[0]:
-            raise IncorrectVMCrecentialsException("No token provided for the InfrastructureManager.")
+            raise InvaliddUserException("No token provided for the InfrastructureManager.")
 
         # First check if an OIDC token is included
         if "token" in im_auth[0]:
@@ -1389,9 +1386,9 @@ class InfrastructureManager:
         elif "username" in im_auth[0]:
             if im_auth[0]['username'].startswith(IM.InfrastructureInfo.InfrastructureInfo.OPENID_USER_PREFIX):
                 # This is a OpenID user do not enable to get data using user/pass creds
-                raise IncorrectVMCrecentialsException("Invalid username used for the InfrastructureManager.")
+                raise InvaliddUserException("Invalid username used for the InfrastructureManager.")
         else:
-            raise IncorrectVMCrecentialsException("No username nor token for the InfrastructureManager.")
+            raise InvaliddUserException("No username nor token for the InfrastructureManager.")
 
         # Now check if the user is in authorized
         if not InfrastructureManager.check_im_user(im_auth):
@@ -1447,7 +1444,8 @@ class InfrastructureManager:
         inf.auth = Authentication(auth.getAuthInfo("InfrastructureManager"))
         IM.InfrastructureList.InfrastructureList.add_infrastructure(inf)
         IM.InfrastructureList.InfrastructureList.save_data(inf.id)
-        InfrastructureManager.logger.info("Creating new Inf ID: " + str(inf.id))
+        im_user = auth.getAuthInfo("InfrastructureManager")[0]['username']
+        InfrastructureManager.logger.info("Creating new Inf ID: %s by %s." % (inf.id, im_user))
 
         # Add the resources in radl_data
         try:
@@ -1487,7 +1485,7 @@ class InfrastructureManager:
             IM.InfrastructureList.InfrastructureList.remove_inf(inf)
             raise e
 
-        InfrastructureManager.logger.info("Inf ID:" + str(inf.id) + ": Successfully created")
+        InfrastructureManager.logger.info("Inf ID: " + str(inf.id) + ": Successfully created")
 
         return inf.id
 
@@ -1605,3 +1603,65 @@ class InfrastructureManager:
     @staticmethod
     def stop():
         IM.InfrastructureList.InfrastructureList.stop()
+
+    @staticmethod
+    def _get_cloud_conn(cloud_id, auth):
+        """
+        Get the CloudConnection object for the specified cloud_id
+
+        Arguments:
+          - cloud_id(string): ID of the cloud provider specified in the Authentication data
+          - auth_data(:py:class:`dict` of str objects): Authentication data to access cloud provider.
+
+        Returns: a CloudConnection object.
+        """
+        cloud_conn = None
+        for cloud in CloudInfo.get_cloud_list(auth):
+            if cloud.id == cloud_id:
+                cloud_conn = cloud.getCloudConnector(None)
+
+        if cloud_conn:
+            return cloud_conn
+        else:
+            InfrastructureManager.logger.error("Cloud ID %s not found." % cloud_id)
+            raise Exception("Cloud ID %s not found." % cloud_id)
+
+    @staticmethod
+    def GetCloudImageList(cloud_id, auth, filters=None):
+        """
+        Get a list of images on the cloud provider using IM URI format.
+
+        Arguments:
+          - cloud_id(string): ID of the cloud provider specified in the Authentication data
+          - auth_data(:py:class:`dict` of str objects): Authentication data to access cloud provider.
+          - filters(:py:class:`dict` of str objects): Pair key value to filter the list of images.
+                                                     It is cloud provider specific.
+
+        Returns: a list dicts with at least two fields "uri" and "name".
+        """
+        # First check the auth data
+        auth = InfrastructureManager.check_auth_data(auth)
+        return InfrastructureManager._get_cloud_conn(cloud_id, auth).list_images(auth)
+
+    @staticmethod
+    def GetCloudQuotas(cloud_id, auth):
+        """
+        Get the number of used and available resources in the cloud provider
+
+        Arguments:
+          - cloud_id(string): ID of the cloud provider specified in the Authentication data
+          - auth_data(:py:class:`dict` of str objects): Authentication data to access cloud provider.
+
+        Returns: dict with the following structure (if there are no limit in some metric, value is set to 1):
+                    {
+                        "cores": {"used": 1, "limit": 10},
+                        "ram": {"used": 1, "limit": 10},
+                        "instances": {"used": 1, "limit": 10},
+                        "floating_ips": {"used": 1, "limit": 10},
+                        "security_groups": {"used": 1, "limit": 10}
+                    }
+                 Some providers may provide more elements.
+        """
+        # First check the auth data
+        auth = InfrastructureManager.check_auth_data(auth)
+        return InfrastructureManager._get_cloud_conn(cloud_id, auth).get_quotas(auth)
