@@ -21,6 +21,7 @@ import string
 import json
 import tempfile
 import logging
+import sys
 from netaddr import IPNetwork, IPAddress
 
 from radl.radl import network, RADL
@@ -96,6 +97,8 @@ class VirtualMachine(LoggerMixin):
         """Message with the cause of the the error in the VM (if known) """
         self.deleting = False
         """Flag to specify that this VM is deletion process"""
+        self.creation_date = int(time.time())
+        """ Creation time of this Inf. """
 
     def serialize(self):
         with self._lock:
@@ -131,6 +134,7 @@ class VirtualMachine(LoggerMixin):
         newvm = VirtualMachine(None, None, None, None, None, None, dic['im_id'])
         # Set creating to False as default to VMs stored with 1.5.5 or old versions
         newvm.creating = False
+        newvm.creation_date = None
         newvm.__dict__.update(dic)
         # If we load a VM that is not configured, set it to False
         # because the configuration process will be lost
@@ -171,8 +175,7 @@ class VirtualMachine(LoggerMixin):
 
         if success:
             self.destroy = True
-        # force the update of the information
-        self.last_update = 0
+            self.last_update = int(time.time())
 
         if not success:
             VirtualMachine.logger.info("Inf ID: " + self.inf.id + ": The VM cannot be finalized: %s" % msg)
@@ -521,6 +524,10 @@ class VirtualMachine(LoggerMixin):
             if self.state == VirtualMachine.FAILED and self.id is None:
                 return False
 
+            if self.destroy:
+                self.state = VirtualMachine.OFF
+                return False
+
             if self.deleting:
                 self.state = VirtualMachine.DELETING
                 return True
@@ -564,8 +571,24 @@ class VirtualMachine(LoggerMixin):
             self.state = new_state
             self.info.systems[0].setValue("state", new_state)
 
+            # Replace the #N# in dns_names
+            self.replace_dns_name(self.info.systems[0])
+
         return updated
 
+    def replace_dns_name(self, vm_system):
+        """Replace the #N# in dns_names."""
+        cont = 0
+        while vm_system.getValue('net_interface.%d.connection' % cont):
+            vm_dns_name = vm_system.getValue('net_interface.%d.dns_name' % cont)
+            # if dns_name is not set in iface 0 set the default one
+            if cont == 0 and vm_dns_name is None:
+                vm_dns_name = Config.DEFAULT_VM_NAME
+            if vm_dns_name and "#N#" in vm_dns_name:
+                vm_dns_name = vm_dns_name.replace("#N#", str(self.im_id))
+                vm_system.setValue('net_interface.%d.dns_name' % cont, vm_dns_name)
+            cont += 1
+
     @staticmethod
     def add_public_net(radl):
         """
@@ -737,7 +760,7 @@ class VirtualMachine(LoggerMixin):
                     vm_system.setValue('net_interface.%s.ip' % num_net, str(private_ip))
                 vm_system.setValue('net_interface.%s.connection' % num_net, private_net.id)
 
-    def get_ssh(self, retry=False):
+    def get_ssh(self, retry=False, auto_close=True):
         """
         Get SSH object to connect with this VM
         """
@@ -754,9 +777,9 @@ class VirtualMachine(LoggerMixin):
             self.log_warn("VM ID %s does not have IP. Do not return SSH Object." % self.im_id)
             return None
         if retry:
-            return SSHRetry(ip, user, passwd, private_key, self.getSSHPort(), proxy_host)
+            return SSHRetry(ip, user, passwd, private_key, self.getSSHPort(), proxy_host, auto_close=auto_close)
         else:
-            return SSH(ip, user, passwd, private_key, self.getSSHPort(), proxy_host)
+            return SSH(ip, user, passwd, private_key, self.getSSHPort(), proxy_host, auto_close=auto_close)
 
     def is_ctxt_process_running(self):
         """ Return the PID of the running process or None if it is not running """
@@ -827,70 +850,77 @@ class VirtualMachine(LoggerMixin):
             self.ctxt_pid = None
             self.configured = False
 
+        ssh = None
         initial_count_out = self.cont_out
         wait = 0
-        while self.ctxt_pid:
-            if self.destroy:
-                # If the VM has been destroyed set pid to None and return
-                self.log_debug("VM %s deleted. Exit check_ctxt_process thread." % self.im_id)
-                self.ctxt_pid = None
-                return None
-
-            ctxt_pid = self.ctxt_pid
-            if ctxt_pid != self.WAIT_TO_PID:
-                ssh = self.get_ssh_ansible_master()
-
-                try:
-                    self.log_info("Getting status of ctxt process with pid: " + str(ctxt_pid))
-                    (_, _, exit_status) = ssh.execute("ps " + str(ctxt_pid))
-                    self.ssh_connect_errors = 0
-                except Exception as ex:
-                    self.log_warn("Error getting status of ctxt process with pid: %s. %s" % (ctxt_pid, ex))
-                    exit_status = 0
-                    self.ssh_connect_errors += 1
-                    if self.ssh_connect_errors > Config.MAX_SSH_ERRORS:
-                        self.log_error("Too much errors getting status of ctxt process with pid: " +
-                                       str(ctxt_pid) + ". Forget it.")
-                        self.ssh_connect_errors = 0
-                        self.configured = False
-                        self.ctxt_pid = None
-                        self.cont_out = initial_count_out + ("Too much errors getting the status of ctxt process."
-                                                             " Check some network connection problems or if user "
-                                                             "credentials has been changed.")
-                        return None
-
-                ip = self.getPublicIP()
-                if not ip:
-                    ip = ip = self.getPrivateIP()
-                remote_dir = "%s/%s/%s_%s" % (Config.REMOTE_CONF_DIR, self.inf.id, ip, self.im_id)
-
-                if exit_status != 0:
-                    # The process has finished, get the outputs
-                    self.log_info("The process %s has finished, get the outputs" % ctxt_pid)
-                    ctxt_log = self.get_ctxt_log(remote_dir, ssh, True)
-                    msg = self.get_ctxt_output(remote_dir, ssh, True)
-                    if ctxt_log:
-                        self.cont_out = initial_count_out + msg + ctxt_log
-                    else:
-                        self.cont_out = initial_count_out + msg + \
-                            "Error getting contextualization process log."
+        try:
+            while self.ctxt_pid:
+                if self.destroy:
+                    # If the VM has been destroyed set pid to None and return
+                    self.log_debug("VM %s deleted. Exit check_ctxt_process thread." % self.im_id)
                     self.ctxt_pid = None
-                else:
-                    # Get the log of the process to update the cont_out
-                    # dynamically
-                    if Config.UPDATE_CTXT_LOG_INTERVAL > 0 and wait > Config.UPDATE_CTXT_LOG_INTERVAL:
-                        wait = 0
-                        self.log_info("Get the log of the ctxt process with pid: " + str(ctxt_pid))
-                        ctxt_log = self.get_ctxt_log(remote_dir, ssh)
-                        self.cont_out = initial_count_out + ctxt_log
-                    # The process is still running, wait
-                    self.log_info("The process %s is still running. wait." % ctxt_pid)
-                    time.sleep(Config.CHECK_CTXT_PROCESS_INTERVAL)
-                    wait += Config.CHECK_CTXT_PROCESS_INTERVAL
-            else:
-                # We are waiting the PID, sleep
-                time.sleep(Config.CHECK_CTXT_PROCESS_INTERVAL)
+                    return None
 
+                ctxt_pid = self.ctxt_pid
+                if ctxt_pid != self.WAIT_TO_PID:
+                    if not ssh:
+                        ssh = self.get_ssh_ansible_master(auto_close=False)
+
+                    try:
+                        self.log_info("Getting status of ctxt process with pid: " + str(ctxt_pid))
+                        (_, _, exit_status) = ssh.execute("ps " + str(ctxt_pid))
+                        self.ssh_connect_errors = 0
+                    except Exception as ex:
+                        self.log_warn("Error getting status of ctxt process with pid: %s. %s" % (ctxt_pid, ex))
+                        exit_status = 0
+                        self.ssh_connect_errors += 1
+                        if self.ssh_connect_errors > Config.MAX_SSH_ERRORS:
+                            self.log_error("Too much errors getting status of ctxt process with pid: " +
+                                           str(ctxt_pid) + ". Forget it.")
+                            self.ssh_connect_errors = 0
+                            self.configured = False
+                            self.ctxt_pid = None
+                            self.cont_out = initial_count_out + ("Too much errors getting the status of ctxt process."
+                                                                 " Check some network connection problems or if user "
+                                                                 "credentials has been changed.")
+                            return None
+
+                    ip = self.getPublicIP()
+                    if not ip:
+                        ip = ip = self.getPrivateIP()
+                    remote_dir = "%s/%s/%s_%s" % (Config.REMOTE_CONF_DIR, self.inf.id, ip, self.im_id)
+
+                    if exit_status != 0:
+                        # The process has finished, get the outputs
+                        self.log_info("The process %s has finished, get the outputs" % ctxt_pid)
+                        ctxt_log = self.get_ctxt_log(remote_dir, ssh, True)
+                        msg = self.get_ctxt_output(remote_dir, ssh, True)
+                        if ctxt_log:
+                            self.cont_out = initial_count_out + msg + ctxt_log
+                        else:
+                            self.cont_out = initial_count_out + msg + \
+                                "Error getting contextualization process log."
+                        self.ctxt_pid = None
+                    else:
+                        # Get the log of the process to update the cont_out
+                        # dynamically
+                        if Config.UPDATE_CTXT_LOG_INTERVAL > 0 and wait > Config.UPDATE_CTXT_LOG_INTERVAL:
+                            wait = 0
+                            self.log_info("Get the log of the ctxt process with pid: " + str(ctxt_pid))
+                            ctxt_log = self.get_ctxt_log(remote_dir, ssh)
+                            self.cont_out = initial_count_out + ctxt_log
+                        # The process is still running, wait
+                        self.log_info("The process %s is still running. wait." % ctxt_pid)
+                        time.sleep(Config.CHECK_CTXT_PROCESS_INTERVAL)
+                        wait += Config.CHECK_CTXT_PROCESS_INTERVAL
+                else:
+                    # We are waiting the PID, sleep
+                    time.sleep(Config.CHECK_CTXT_PROCESS_INTERVAL)
+        except Exception as gex:
+            self.log_warn("Error getting status of ctxt process with pid: %s. %s" % (self.ctxt_pid, gex))
+        finally:
+            if ssh:
+                ssh.close()
         return self.ctxt_pid
 
     def is_configured(self):
@@ -906,19 +936,23 @@ class VirtualMachine(LoggerMixin):
 
     def get_ctxt_log(self, remote_dir, ssh, delete=False):
         tmp_dir = tempfile.mkdtemp()
-        conf_out = ""
 
         # Download the contextualization agent log
         try:
             # Get the messages of the contextualization process
             self.log_debug("Get File: " + remote_dir + '/ctxt_agent.log')
             ssh.sftp_get(remote_dir + '/ctxt_agent.log', tmp_dir + '/ctxt_agent.log')
-            with open(tmp_dir + '/ctxt_agent.log') as f:
-                conf_out = f.read()
 
-            # Remove problematic chars
-            conf_out = str("".join(list(filter(lambda x: x in string.printable,
-                                               conf_out))).encode("ascii", "replace").decode("utf-8"))
+            # patch for support python2 and avoid UnicodeDecodeError in python3
+            if sys.version_info[0] < 3:
+                open_args = {}
+            else:
+                open_args = {'encoding': 'utf-8'}
+
+            with open(tmp_dir + '/ctxt_agent.log', **open_args) as f:
+                # Read removing problematic chars
+                conf_out = str("".join(list(filter(lambda x: x in string.printable,
+                                                   f.read()))).encode("ascii", "replace").decode("utf-8"))
             try:
                 if delete:
                     ssh.sftp_remove(remote_dir + '/ctxt_agent.log')
@@ -926,6 +960,7 @@ class VirtualMachine(LoggerMixin):
                 self.log_exception(
                     "Error deleting remote contextualization process log: " + remote_dir + '/ctxt_agent.log')
         except Exception:
+            conf_out = ""
             self.log_exception(
                 "Error getting contextualization process log: " + remote_dir + '/ctxt_agent.log')
             self.configured = False
@@ -1011,17 +1046,17 @@ class VirtualMachine(LoggerMixin):
 
         return ansible_host
 
-    def get_ssh_ansible_master(self, retry=True):
+    def get_ssh_ansible_master(self, retry=True, auto_close=True):
         ansible_host = self.get_ansible_host()
         if ansible_host:
             (user, passwd, private_key) = ansible_host.getCredentialValues()
             if retry:
-                return SSHRetry(ansible_host.getHost(), user, passwd, private_key)
+                return SSHRetry(ansible_host.getHost(), user, passwd, private_key, auto_close=auto_close)
             else:
-                return SSH(ansible_host.getHost(), user, passwd, private_key)
+                return SSH(ansible_host.getHost(), user, passwd, private_key, auto_close=auto_close)
         else:
             if self.inf.vm_master:
-                return self.inf.vm_master.get_ssh(retry=retry)
+                return self.inf.vm_master.get_ssh(retry=retry, auto_close=auto_close)
             else:
                 self.log_warn("There is not master VM. Do not return SSH object.")
                 return None
