@@ -42,6 +42,80 @@ except Exception as ex:
     print(ex)
 
 
+class AzureInstanceTypeInfo:
+    """
+    Information about the instance type
+
+    Args:
+            - name(str, optional): name of the type of the instance
+            - cpu(int, optional): number of cpus
+            - mem(int, optional): amount of memory
+            - disk_space(int, optional): size of the disks
+            - gpu(int, optional): the number of gpus of this instance
+            - gpu_model(str, optional): the model of the gpus of this instance
+            - gpu_vendor(str, optional): the model of the gpus of this instance
+    """
+
+    def __init__(self, name="", cpu=1, mem=0, os_disk_space=0, res_disk_space=0,
+                 gpu=0, gpu_model=None, gpu_vendor=None):
+        self.name = name
+        self.cpu = cpu
+        self.mem = mem
+        self.os_disk_space = os_disk_space
+        self.res_disk_space = res_disk_space
+        self.gpu = gpu
+        self.gpu_model = gpu_model
+        self.gpu_vendor = gpu_vendor
+
+    def set_gpu_models(self):
+        """Guess GPU models from instance name"""
+        if self.name.startswith('Standard_NC'):
+            self.gpu_vendor = "NVIDIA"
+            if self.name.endswith('v2'):
+                self.gpu_model = "Tesla P100"
+            elif self.name.endswith('v3'):
+                self.gpu_model = "Tesla V100"
+            else:
+                self.gpu_model = "Tesla K80"
+        elif self.name.startswith('Standard_NCasT4'):
+            self.gpu_vendor = "NVIDIA"
+            self.gpu_model = "Tesla T4"
+        elif self.name.startswith('Standard_ND'):
+            self.gpu_vendor = "NVIDIA"
+            if self.name.endswith('v2'):
+                self.gpu_model = "Tesla V100"
+            else:
+                self.gpu_model = "Tesla P40"
+        elif self.name.startswith('Standard_NV'):
+            self.gpu_vendor = "NVIDIA"
+            if self.name.endswith('v3'):
+                self.gpu_model = "Tesla M60"
+            elif self.name.endswith('v4'):
+                self.gpu_vendor = "AMD"
+                self.gpu_model = "Radeon instinto MI25"
+            else:
+                self.gpu_model = "Tesla M60"
+
+    @staticmethod
+    def fromSKU(sku):
+        """Get an instance type object from SKU Json data"""
+        gpu = os_disk_space = res_disk_space = mem = cpu = 0
+        for elem in sku.capabilities:
+            if elem.name == "vCPUs":
+                cpu = int(elem.value)
+            elif elem.name == "MemoryGB":
+                mem = float(elem.value) * 1024
+            elif elem.name == "MaxResourceVolumeMB":
+                res_disk_space = int(elem.value)
+            elif elem.name == "OSVhdSizeMB":
+                os_disk_space = int(elem.value)
+            elif elem.name == "GPUs":
+                gpu = int(elem.value)
+        instance_type = AzureInstanceTypeInfo(sku.name, cpu, mem, os_disk_space, res_disk_space, gpu)
+        instance_type.set_gpu_models()
+        return instance_type
+
+
 class AzureCloudConnector(CloudConnector):
     """
     Cloud Launcher to the Azure platform
@@ -119,10 +193,8 @@ class AzureCloudConnector(CloudConnector):
 
         return self.credentials, subscription_id
 
-    @staticmethod
-    def get_instance_type_by_name(instance_name, location, credentials, subscription_id):
-        compute_client = ComputeManagementClient(credentials, subscription_id)
-        instace_types = compute_client.virtual_machine_sizes.list(location)
+    def get_instance_type_by_name(self, instance_name, location, credentials, subscription_id):
+        instace_types = self.get_instance_type_list(credentials, subscription_id, location)
 
         for instace_type in list(instace_types):
             if instace_type.name == instance_name:
@@ -130,6 +202,17 @@ class AzureCloudConnector(CloudConnector):
 
         return None
 
+    @staticmethod
+    def get_instance_type_list(credentials, subscription_id, location):
+        compute_client = ComputeManagementClient(credentials, subscription_id)
+
+        skus = list(compute_client.resource_skus.list(filter="location eq '%s'" % location))
+        instance_types = [AzureInstanceTypeInfo.fromSKU(sku) for sku in skus if sku.resource_type == "virtualMachines"]
+
+        instance_types.sort(key=lambda x: (x.cpu, x.mem, x.gpu, x.res_disk_space))
+
+        return instance_types
+
     def get_instance_type(self, system, credentials, subscription_id):
         """
         Get the name of the instance type to launch to Azure
@@ -145,19 +228,28 @@ class AzureCloudConnector(CloudConnector):
             location = system.getValue('availability_zone')
 
         (cpu, cpu_op, memory, memory_op, disk_free, disk_free_op) = self.get_instance_selectors(system)
+        num_gpus = system.getValue('gpu.count')
+        gpu_model = system.getValue('gpu.model')
+        gpu_vendor = system.getValue('gpu.vendor')
 
-        compute_client = ComputeManagementClient(credentials, subscription_id)
-        instace_types = list(compute_client.virtual_machine_sizes.list(location))
-        instace_types.sort(key=lambda x: (x.number_of_cores, x.memory_in_mb, x.resource_disk_size_in_mb))
+        instace_types = self.get_instance_type_list(credentials, subscription_id, location)
 
         default = None
         for instace_type in instace_types:
             if instace_type.name == self.INSTANCE_TYPE:
                 default = instace_type
 
-            comparison = cpu_op(instace_type.number_of_cores, cpu)
-            comparison = comparison and memory_op(instace_type.memory_in_mb, memory)
-            comparison = comparison and disk_free_op(instace_type.resource_disk_size_in_mb, disk_free)
+            comparison = cpu_op(instace_type.cpu, cpu)
+            comparison = comparison and memory_op(instace_type.mem, memory)
+            comparison = comparison and disk_free_op(instace_type.res_disk_space, disk_free)
+
+            if num_gpus:
+                if num_gpus > instace_type.gpu:
+                    continue
+                if gpu_vendor and gpu_vendor.lower() != instace_type.gpu_vendor.lower():
+                    return False
+                if gpu_model and gpu_model.lower() != instace_type.gpu_model.lower():
+                    return False
 
             if comparison:
                 if not instance_type_name or instace_type.name == instance_type_name:
@@ -170,14 +262,23 @@ class AzureCloudConnector(CloudConnector):
         """
         Update the features of the system with the information of the instance_type
         """
-        system.addFeature(Feature("cpu.count", "=", instance_type.number_of_cores),
+        system.addFeature(Feature("cpu.count", "=", instance_type.cpu),
                           conflict="other", missing="other")
-        system.addFeature(Feature("memory.size", "=", instance_type.memory_in_mb, 'M'),
+        system.addFeature(Feature("memory.size", "=", instance_type.mem, 'M'),
                           conflict="other", missing="other")
-        system.addFeature(Feature("disks.free_size", "=", instance_type.resource_disk_size_in_mb, 'M'),
+        system.addFeature(Feature("disks.free_size", "=", instance_type.res_disk_space, 'M'),
                           conflict="other", missing="other")
         system.addFeature(Feature("instance_type", "=", instance_type.name),
                           conflict="other", missing="other")
+        if instance_type.gpu:
+            system.addFeature(Feature("gpu.count", "=", instance_type.gpu),
+                              conflict="other", missing="other")
+        if instance_type.gpu_model:
+            system.addFeature(Feature("gpu.model", "=", instance_type.gpu_model),
+                              conflict="other", missing="other")
+        if instance_type.gpu_vendor:
+            system.addFeature(Feature("gpu.vendor", "=", instance_type.gpu_vendor),
+                              conflict="other", missing="other")
 
     def concrete_system(self, radl_system, str_url, auth_data):
         url = urlparse(str_url)
@@ -382,7 +483,7 @@ class AzureCloudConnector(CloudConnector):
         return res
 
     def get_azure_vm_create_json(self, group_name, storage_account, vm_name, nics, radl,
-                                 instance_type, custom_data, compute_client):
+                                 instance_type, custom_data, compute_client, tags):
         """ Create the VM parameters structure. """
         system = radl.systems[0]
         url = urlparse(system.getValue("disk.0.image.url"))
@@ -471,7 +572,6 @@ class AzureCloudConnector(CloudConnector):
                 'custom_data': custom_data
             }
 
-        tags = self.get_instance_tags(system)
         if tags:
             vm['tags'] = tags
 
@@ -561,7 +661,7 @@ class AzureCloudConnector(CloudConnector):
         return subnets
 
     def create_vms(self, inf, radl, requested_radl, num_vm, location, storage_account_name,
-                   subnets, credentials, subscription_id):
+                   subnets, credentials, subscription_id, tags):
         """
         Creates a set of VMs
         """
@@ -569,26 +669,10 @@ class AzureCloudConnector(CloudConnector):
         vms = []
         i = 0
         while i < num_vm:
-            uid = str(uuid.uuid1())
-
-            vm_name = radl.systems[0].getValue("instance_name")
-            if vm_name:
-                vm_name = "%s-%s" % (vm_name, uid)
-            else:
-                vm_name = "userimage-%s" % uid
-
+            vm_name = self.gen_instance_name(radl.systems[0])
             group_name = "rg-%s" % (vm_name)
 
             try:
-                tags = {}
-                if radl.systems[0].getValue('instance_tags'):
-                    keypairs = radl.systems[0].getValue('instance_tags').split(",")
-                    for keypair in keypairs:
-                        parts = keypair.split("=")
-                        key = parts[0].strip()
-                        value = parts[1].strip()
-                        tags[key] = value
-
                 args = {'location': location}
                 if tags:
                     args['tags'] = tags
@@ -606,7 +690,7 @@ class AzureCloudConnector(CloudConnector):
                 instance_type = self.get_instance_type(radl.systems[0], credentials, subscription_id)
                 vm_parameters = self.get_azure_vm_create_json(group_name, storage_account_name, vm_name,
                                                               nics, radl, instance_type, custom_data,
-                                                              compute_client)
+                                                              compute_client, tags)
 
                 async_vm_creation = compute_client.virtual_machines.create_or_update(group_name,
                                                                                      vm_name,
@@ -660,6 +744,8 @@ class AzureCloudConnector(CloudConnector):
         resource_client = ResourceManagementClient(credentials, subscription_id)
         storage_account_name = self.get_storage_account_name(inf.id)
 
+        tags = self.get_instance_tags(radl.systems[0], auth_data, inf)
+
         with inf._lock:
             # Create resource group for the Infrastructure if it does not exists
             if not self.get_rg("rg-%s" % inf.id, credentials, subscription_id):
@@ -688,7 +774,7 @@ class AzureCloudConnector(CloudConnector):
 
         res = []
         vms = self.create_vms(inf, radl, requested_radl, num_vm, location,
-                              storage_account_name, subnets, credentials, subscription_id)
+                              storage_account_name, subnets, credentials, subscription_id, tags)
 
         remaining_vms = num_vm
         for success, data in vms:
@@ -767,16 +853,12 @@ class AzureCloudConnector(CloudConnector):
            - credentials, subscription_id: Authentication data to access cloud provider.
         """
         try:
-            group_name = vm.id.split('/')[0]
-            dns_client = DnsManagementClient(credentials, subscription_id)
-            system = vm.info.systems[0]
-            for net_name in system.getNetworkIDs():
-                num_conn = system.getNumNetworkWithConnection(net_name)
-                ip = system.getIfaceIP(num_conn)
-                (hostname, domain) = vm.getRequestedNameIface(num_conn,
-                                                              default_hostname=Config.DEFAULT_VM_NAME,
-                                                              default_domain=Config.DEFAULT_DOMAIN)
-                if domain != "localdomain" and ip:
+            dns_entries = self.get_dns_entries(vm)
+            if dns_entries:
+                group_name = vm.id.split('/')[0]
+                dns_client = DnsManagementClient(credentials, subscription_id)
+                for hostname, domain, ip in dns_entries:
+                    domain = domain[:-1]
                     zone = None
                     try:
                         zone = dns_client.zones.get(group_name, domain)
@@ -951,3 +1033,34 @@ class AzureCloudConnector(CloudConnector):
             self.log_info("Resource group %s successfully deleted." % group_name)
 
         return deleted, msg
+
+    def list_images(self, auth_data, filters=None):
+        location = self.DEFAULT_LOCATION
+        publisher = ['Canonical', 'MicrosoftSQLServer', 'MicrosoftWindowsDesktop',
+                     'MicrosoftWindowsServer', 'nvidia', 'Oracle', 'RedHat', 'SUSE']
+        if filters and 'location' in filters and filters['location']:
+            location = filters['location']
+        if filters and 'publisher' in filters and filters['publisher']:
+            publisher = filters['publisher'].split(",")
+
+        credentials, subscription_id = self.get_credentials(auth_data)
+        compute_client = ComputeManagementClient(credentials, subscription_id)
+
+        # If publisher is "*" it means all
+        if publisher == ["*"]:
+            pubs = compute_client.virtual_machine_images.list_publishers(location)
+            publisher = [pub.name for pub in pubs]
+
+        images = []
+        for pub in publisher:
+            offers = compute_client.virtual_machine_images.list_offers(location, pub)
+            for offer in offers:
+                skus = compute_client.virtual_machine_images.list_skus(location, pub, offer.name)
+                for sku in skus:
+                    images.append((pub, offer.name, sku.name))
+
+        res = []
+        for pub, offer, sku in images:
+            res.append({"uri": "azr://%s/%s/%s/latest" % (pub, offer, sku),
+                               "name": "%s %s %s" % (pub, offer, sku)})
+        return res
