@@ -97,7 +97,7 @@ class CloudStackCloudConnector(LibCloudCloudConnector):
             driver = self.get_driver(auth_data)
 
             res_system = radl_system.clone()
-            instance_type = self.get_instance_type(driver.list_sizes(), res_system)
+            instance_type = self.get_instance_type(driver, res_system)
             self.update_system_info_from_instance(res_system, instance_type)
 
             username = res_system.getValue('disk.0.os.credentials.username')
@@ -207,13 +207,7 @@ class CloudStackCloudConnector(LibCloudCloudConnector):
         image_id = self.get_image_id(system.getValue("disk.0.image.url"))
         image = NodeImage(id=image_id, name=None, driver=driver)
 
-        instance_type = self.get_instance_type(driver.list_sizes(), system)
-
-        name = system.getValue("instance_name")
-        if not name:
-            name = system.getValue("disk.0.image.name")
-        if not name:
-            name = "userimage"
+        instance_type = self.get_instance_type(driver, system)
 
         sgs = self.create_security_groups(driver, inf, radl)
 
@@ -221,7 +215,7 @@ class CloudStackCloudConnector(LibCloudCloudConnector):
                 'image': image,
                 'ex_security_groups': sgs,
                 'ex_start_vm': True,
-                'name': "%s-%s" % (name, str(uuid.uuid1()))}
+                'name': self.gen_instance_name(system)}
 
         if system.getValue('availability_zone'):
             args['location'] = system.getValue('availability_zone')
@@ -230,8 +224,8 @@ class CloudStackCloudConnector(LibCloudCloudConnector):
         public_key = system.getValue("disk.0.os.credentials.public_key")
 
         if public_key and public_key.find('-----BEGIN CERTIFICATE-----') == -1:
-            public_key = None
             keypair = driver.get_key_pair(public_key)
+            public_key = None
             if keypair:
                 system.setUserKeyCredentials(system.getCredentials().username, None, keypair.private_key)
             else:
@@ -245,7 +239,7 @@ class CloudStackCloudConnector(LibCloudCloudConnector):
             user = self.DEFAULT_USER
             system.setValue('disk.0.os.credentials.username', user)
 
-        tags = self.get_instance_tags(system)
+        tags = self.get_instance_tags(system, auth_data, inf)
 
         res = []
         i = 0
@@ -425,8 +419,7 @@ class CloudStackCloudConnector(LibCloudCloudConnector):
     def alterVM(self, vm, radl, auth_data):
         node = self.get_node_with_id(vm.id, auth_data)
         if node:
-            instance_type = self.get_instance_type(
-                node.driver.list_sizes(), radl.systems[0])
+            instance_type = self.get_instance_type(node.driver, radl.systems[0])
 
             try:
                 if node.ex_stop() == "Stopped":
