@@ -38,7 +38,6 @@ except ImportError:
     from urllib.parse import urlparse
 from IM.VirtualMachine import VirtualMachine
 from radl.radl import Feature
-from IM.config import Config
 
 
 class GCECloudConnector(LibCloudCloudConnector):
@@ -154,7 +153,7 @@ class GCECloudConnector(LibCloudCloudConnector):
             else:
                 region, _ = self.get_image_data(str_url)
 
-            instance_type = self.get_instance_type(driver.list_sizes(region), res_system)
+            instance_type = self.get_instance_type(driver, res_system, region)
             if not instance_type:
                 return None
 
@@ -176,8 +175,14 @@ class GCECloudConnector(LibCloudCloudConnector):
         if isinstance(instance_type, NodeSize):
             LibCloudCloudConnector.update_system_info_from_instance(system, instance_type)
             if 'guestCpus' in instance_type.extra:
-                system.addFeature(Feature("cpu.count", "=", instance_type.extra[
-                                  'guestCpus']), conflict="other", missing="other")
+                system.addFeature(Feature("cpu.count", "=", instance_type.extra['guestCpus']),
+                                  conflict="other", missing="other")
+            if 'accelerators' in instance_type.extra and instance_type.extra['accelerators']:
+                gpu_info = instance_type.extra['accelerators'][0]
+                system.addFeature(Feature("gpu.count", "=", gpu_info['guestAcceleratorCount']),
+                                  conflict="other", missing="other")
+                system.addFeature(Feature("gpu.model", "=", gpu_info['guestAcceleratorType']),
+                                  conflict="other", missing="other")
 
     @staticmethod
     def set_net_provider_id(radl, net_name):
@@ -210,18 +215,29 @@ class GCECloudConnector(LibCloudCloudConnector):
 
         return provider_id
 
-    def get_instance_type(self, sizes, radl):
-        """
-        Get the name of the instance type to launch to LibCloud
+    @staticmethod
+    def guess_instance_type_gpu(size, num_gpus, vendor=None, model=None):
+        """Try to guess if this NodeSize has GPU support"""
+        if 'accelerators' in size.extra and size.extra['accelerators']:
+            for accelerator in size.extra['accelerators']:
+                if num_gpus < accelerator['guestAcceleratorCount']:
+                    continue
+                if model and model.lower() not in accelerator['guestAcceleratorType'].lower():
+                    continue
+                if vendor and vendor.lower() not in accelerator['guestAcceleratorType'].lower():
+                    continue
+                return True
 
-        Arguments:
-           - size(list of :py:class: `libcloud.compute.base.NodeSize`): List of sizes on a provider
-           - radl(str): RADL document with the requirements of the VM to get the instance type
-        Returns: a :py:class:`libcloud.compute.base.NodeSize` with the instance type to launch
-        """
+        return False
+
+    def get_instance_type(self, driver, radl, location=None):
+        sizes = driver.list_sizes(location)
         instance_type_name = radl.getValue('instance_type')
 
         (cpu, cpu_op, memory, memory_op, _, _) = self.get_instance_selectors(radl)
+        num_gpus = radl.getValue('gpu.count')
+        gpu_model = radl.getValue('gpu.model')
+        gpu_vendor = radl.getValue('gpu.vendor')
 
         res = None
         for size in sizes:
@@ -234,6 +250,9 @@ class GCECloudConnector(LibCloudCloudConnector):
                 if 'guestCpus' in size.extra and size.extra['guestCpus']:
                     comparison = comparison and cpu_op(size.extra['guestCpus'], cpu)
 
+                if num_gpus and not self.guess_instance_type_gpu(size, num_gpus, gpu_vendor, gpu_model):
+                    continue
+
                 if comparison:
                     if not instance_type_name or size.name == instance_type_name:
                         res = size
@@ -510,25 +529,23 @@ class GCECloudConnector(LibCloudCloudConnector):
         if not image:
             return [(False, "Incorrect image name") for _ in range(num_vm)]
 
-        instance_type = self.get_instance_type(driver.list_sizes(region), system)
+        instance_type = self.get_instance_type(driver, system, region)
 
         if not instance_type:
             raise Exception("No compatible size found")
 
-        name = system.getValue("instance_name")
-        if not name:
-            name = system.getValue("disk.0.image.name")
-        if not name:
-            name = "userimage"
+        name = self.gen_instance_name(system)
 
         args = {'size': instance_type,
                 'image': image,
                 'external_ip': None,
                 'location': region}
 
-        tags = self.get_instance_tags(system)
+        tags = self.get_instance_tags(system, auth_data, inf)
         if tags:
-            args['ex_metadata'] = tags
+            args['ex_labels'] = {}
+            for key, value in tags.items():
+                args['ex_labels'][key.replace("-", "_").lower()] = value
 
         # include the SSH_KEYS
         username = system.getValue('disk.0.os.credentials.username')
@@ -587,7 +604,7 @@ class GCECloudConnector(LibCloudCloudConnector):
         error_msg = "Error launching VM."
         if num_vm > 1:
             args['number'] = num_vm
-            args['base_name'] = "%s-%s" % (name.lower().replace("_", "-"), str(uuid.uuid1()))
+            args['base_name'] = name
             self.log_debug(args)
             try:
                 nodes = driver.ex_create_multiple_nodes(**args)
@@ -596,7 +613,7 @@ class GCECloudConnector(LibCloudCloudConnector):
                 self.log_exception("Error launching VMs.")
                 error_msg = str(ex)
         else:
-            args['name'] = "%s-%s" % (name.lower().replace("_", "-"), str(uuid.uuid1()))
+            args['name'] = name
             self.log_debug(args)
             try:
                 nodes = [driver.create_node(**args)]
@@ -850,17 +867,10 @@ class GCECloudConnector(LibCloudCloudConnector):
            - auth_data(:py:class:`dict` of str objects): Authentication data to access cloud provider.
         """
         try:
-            driver = self.get_dns_driver(auth_data)
-            system = vm.info.systems[0]
-            for net_name in system.getNetworkIDs():
-                num_conn = system.getNumNetworkWithConnection(net_name)
-                ip = system.getIfaceIP(num_conn)
-                (hostname, domain) = vm.getRequestedNameIface(num_conn,
-                                                              default_hostname=Config.DEFAULT_VM_NAME,
-                                                              default_domain=Config.DEFAULT_DOMAIN)
-                if domain != "localdomain" and ip:
-                    if not domain.endswith("."):
-                        domain += "."
+            dns_entries = self.get_dns_entries(vm)
+            if dns_entries:
+                driver = self.get_dns_driver(auth_data)
+                for hostname, domain, ip in dns_entries:
                     zone = [z for z in driver.iterate_zones() if z.domain == domain]
                     if not zone:
                         self.log_info("Creating DNS zone %s" % domain)
@@ -892,17 +902,10 @@ class GCECloudConnector(LibCloudCloudConnector):
            - auth_data(:py:class:`dict` of str objects): Authentication data to access cloud provider.
         """
         try:
-            driver = self.get_dns_driver(auth_data)
-            system = vm.info.systems[0]
-            for net_name in system.getNetworkIDs():
-                num_conn = system.getNumNetworkWithConnection(net_name)
-                ip = system.getIfaceIP(num_conn)
-                (hostname, domain) = vm.getRequestedNameIface(num_conn,
-                                                              default_hostname=Config.DEFAULT_VM_NAME,
-                                                              default_domain=Config.DEFAULT_DOMAIN)
-                if domain != "localdomain" and ip:
-                    if not domain.endswith("."):
-                        domain += "."
+            dns_entries = self.get_dns_entries(vm)
+            if dns_entries:
+                driver = self.get_dns_driver(auth_data)
+                for hostname, domain, ip in dns_entries:
                     zone = [z for z in driver.iterate_zones() if z.domain == domain]
                     if not zone:
                         self.log_info("The DNS zone %s does not exists. Do not delete records." % domain)
@@ -968,3 +971,22 @@ class GCECloudConnector(LibCloudCloudConnector):
 
     def alterVM(self, vm, radl, auth_data):
         return (False, "Not supported")
+
+    def list_images(self, auth_data, filters=None):
+        driver = self.get_driver(auth_data)
+        images = []
+        gce_images = driver.list_images()
+
+        region = None
+        auth = auth_data.getAuthInfo(self.type)[0]
+        if 'region' in auth and auth['region']:
+            region = auth['region']
+        if filters and 'region' in filters and filters['region']:
+            region = filters['region']
+
+        for location in driver.list_locations():
+            if not region or region == location.name:
+                for image in gce_images:
+                    images.append({"uri": "gce://%s/%s" % (location.name, image.id),
+                                   "name": "%s/%s" % (location.name, image.name)})
+        return images
