@@ -49,7 +49,8 @@ class LibCloudCloudConnector(CloudConnector):
         NodeState.PENDING: VirtualMachine.PENDING,
         NodeState.TERMINATED: VirtualMachine.OFF,
         NodeState.STOPPED: VirtualMachine.STOPPED,
-        NodeState.ERROR: VirtualMachine.FAILED
+        NodeState.ERROR: VirtualMachine.FAILED,
+        NodeState.UNKNOWN: VirtualMachine.UNKNOWN
     }
     """State map"""
 
@@ -103,15 +104,17 @@ class LibCloudCloudConnector(CloudConnector):
                 self.log_error("Incorrect auth data")
                 return None
 
-    def get_instance_type(self, sizes, radl):
+    def get_instance_type(self, driver, radl, location=None):
         """
         Get the name of the instance type to launch to LibCloud
 
         Arguments:
-           - size(list of :py:class: `libcloud.compute.base.NodeSize`): List of sizes on a provider
+           - driver(:py:class:`libcloud.compute.base.NodeDriver`): Driver to use.
            - radl(str): RADL document with the requirements of the VM to get the instance type
+           - location:(:class:`.NodeLocation`): optional location.
         Returns: a :py:class:`libcloud.compute.base.NodeSize` with the instance type to launch
         """
+        sizes = driver.list_sizes()
         instance_type_name = radl.getValue('instance_type')
 
         (_, _, memory, memory_op, disk_free, disk_free_op) = self.get_instance_selectors(radl, disk_unit='G')
@@ -143,7 +146,7 @@ class LibCloudCloudConnector(CloudConnector):
 
         if req_protocol is None or protocol == req_protocol:
             res_system = radl_system.clone()
-            instance_type = self.get_instance_type(driver.list_sizes(), res_system)
+            instance_type = self.get_instance_type(driver, res_system)
             self.update_system_info_from_instance(res_system, instance_type)
             return res_system
         else:
@@ -196,17 +199,11 @@ class LibCloudCloudConnector(CloudConnector):
         image_id = self.get_image_id(system.getValue("disk.0.image.url"))
         image = NodeImage(id=image_id, name=None, driver=driver)
 
-        instance_type = self.get_instance_type(driver.list_sizes(), system)
-
-        name = system.getValue("instance_name")
-        if not name:
-            name = system.getValue("disk.0.image.name")
-        if not name:
-            name = "userimage"
+        instance_type = self.get_instance_type(driver, system)
 
         args = {'size': instance_type,
                 'image': image,
-                'name': "%s-%s" % (name, str(uuid.uuid1()))}
+                'name': self.gen_instance_name(system)}
 
         keypair = None
         public_key = system.getValue("disk.0.os.credentials.public_key")
@@ -460,32 +457,22 @@ class LibCloudCloudConnector(CloudConnector):
     def start(self, vm, auth_data):
         node = self.get_node_with_id(vm.id, auth_data)
         if node:
-            func = getattr(node.driver, "ex_start_node",
-                           getattr(node.driver, "ex_resume_node", None))
-            if func:
-                success = func(node)
-                if success:
-                    return (True, "")
-                else:
-                    return (False, "Error in start operations")
+            success = node.start()
+            if success:
+                return (True, "")
             else:
-                return (False, "Not supported")
+                return (False, "Error in stop operation")
         else:
             return (False, "VM not found with id: " + vm.id)
 
     def stop(self, vm, auth_data):
         node = self.get_node_with_id(vm.id, auth_data)
         if node:
-            func = getattr(node.driver, "ex_stop_node",
-                           getattr(node.driver, "ex_suspend_node", None))
-            if func:
-                success = func(node)
-                if success:
-                    return (True, "")
-                else:
-                    return (False, "Error in stop operation")
+            success = node.stop_node()
+            if success:
+                return (True, "")
             else:
-                return (False, "Not supported")
+                return (False, "Error in stop operation")
         else:
             return (False, "VM not found with id: " + vm.id)
 
