@@ -67,6 +67,10 @@ class TEMPLATE(XMLObject):
     noneval = 0
 
 
+class USER_TEMPLATE(XMLObject):
+    values = ['ERROR']
+
+
 class HISTORY(XMLObject):
     values = ['SEQ', 'HOSTNAME', 'HID', 'STIME', 'ETIME', 'PSTIME',
               'PETIME', 'RSTIME', 'RETIME', 'ESTIME', 'EETIME', 'REASON']
@@ -84,7 +88,7 @@ class VM(XMLObject):
     STATE_POWEROFF = 8
     values = ['ID', 'UID', 'NAME', 'LAST_POLL', 'STATE', 'LCM_STATE',
               'DEPLOY_ID', 'MEMORY', 'CPU', 'NET_TX', 'NET_RX', 'STIME', 'ETIME']
-    tuples = {'TEMPLATE': TEMPLATE}
+    tuples = {'TEMPLATE': TEMPLATE, 'USER_TEMPLATE': USER_TEMPLATE}
     numeric = ['ID', 'UID', 'STATE', 'LCM_STATE', 'STIME', 'ETIME']
 
 
@@ -127,6 +131,7 @@ class VNET_POOL(XMLObject):
 
 class IMAGE(XMLObject):
     STATE_READY = 1
+    STATE_USED = 2
     STATE_ERROR = 5
     values = ['ID', 'UID', 'GID', 'UNAME', 'GNAME', 'NAME', 'SOURCE', 'PATH'
               'FSTYPE', 'TYPE', 'DISK_TYPE', 'PERSISTENT', 'SIZE', 'STATE']
@@ -156,6 +161,57 @@ class SECURITY_GROUP_POOL(XMLObject):
     tuples_lists = {'SECURITY_GROUP': SECURITY_GROUP}
 
 
+class DATASTORE_QUOTA_ITEM(XMLObject):
+    values = ['ID', 'IMAGES', 'IMAGES_USED', 'SIZE', 'SIZE_USED']
+    numeric = ['ID', 'IMAGES', 'IMAGES_USED', 'SIZE', 'SIZE_USED']
+
+
+class DATASTORE_QUOTA(XMLObject):
+    tuples_lists = {'DATASTORE': DATASTORE_QUOTA_ITEM}
+
+
+class NETWORK_QUOTA_ITEM(XMLObject):
+    values = ['ID', 'LEASES', 'LEASES_USED']
+    numeric = ['LEASES', 'LEASES_USED']
+
+
+class NETWORK_QUOTA(XMLObject):
+    tuples_lists = {'NETWORK': NETWORK_QUOTA_ITEM}
+
+
+class VM_QUOTA_ITEM(XMLObject):
+    values = ['CPU', 'CPU_USED', 'MEMORY', 'MEMORY_USED', 'RUNNING_CPU', 'RUNNING_CPU_USED',
+              'RUNNING_MEMORY', 'RUNNING_MEMORY_USED', 'RUNNING_VMS', 'RUNNING_VMS_USED',
+              'SYSTEM_DISK_SIZE', 'SYSTEM_DISK_SIZE_USED', 'VMS', 'VMS_USED']
+    numeric = ['CPU', 'CPU_USED', 'MEMORY', 'MEMORY_USED', 'RUNNING_CPU', 'RUNNING_CPU_USED',
+               'RUNNING_MEMORY', 'RUNNING_MEMORY_USED', 'RUNNING_VMS', 'RUNNING_VMS_USED',
+               'SYSTEM_DISK_SIZE', 'SYSTEM_DISK_SIZE_USED', 'VMS', 'VMS_USED']
+
+
+class VM_QUOTA(XMLObject):
+    tuples = {'VM': VM_QUOTA_ITEM}
+
+
+class IMAGE_QUOTA_ITEM(XMLObject):
+    values = ['ID', 'RVMS', 'RVMS_USED']
+    numeric = ['ID', 'RVMS', 'RVMS_USED']
+
+
+class IMAGE_QUOTA(XMLObject):
+    tuples_lists = {'IMAGE': IMAGE_QUOTA_ITEM}
+
+
+class DEFAULT_USER_QUOTAS(XMLObject):
+    tuples = {'DATASTORE_QUOTA': DATASTORE_QUOTA, 'NETWORK_QUOTA': NETWORK_QUOTA,
+              'VM_QUOTA': VM_QUOTA, 'IMAGE_QUOTA': IMAGE_QUOTA}
+
+
+class USER(XMLObject):
+    values = ['ID', 'GID', 'GNAME', 'NAME']
+    tuples = {'DATASTORE_QUOTA': DATASTORE_QUOTA, 'NETWORK_QUOTA': NETWORK_QUOTA, 'VM_QUOTA': VM_QUOTA,
+              'IMAGE_QUOTA': IMAGE_QUOTA, 'DEFAULT_USER_QUOTAS': DEFAULT_USER_QUOTAS}
+
+
 class OpenNebulaCloudConnector(CloudConnector):
     """
     Cloud Launcher to the OpenNebula platform
@@ -192,7 +248,7 @@ class OpenNebulaCloudConnector(CloudConnector):
                 disk_free = int(radl_system.getFeature('disks.free_size').getValue('M'))
                 # The VMRC specified the value in MB
                 if radl_system.getValue("disk.0.size"):
-                    disk_size = int(radl_system.getValue("disk.0.size"))
+                    disk_size = radl_system.getFeature("disk.0.size").getValue('M')
                 else:
                     disk_size = 0
 
@@ -233,7 +289,8 @@ class OpenNebulaCloudConnector(CloudConnector):
             return auth['username'] + ":" + passwd
         elif 'token' in auth:
             username, passwd = ONETTSClient.get_auth_from_tts(ConfigOpenNebula.TTS_URL,
-                                                              self.cloud.server, auth['token'])
+                                                              self.cloud.server, auth['token'],
+                                                              Config.VERIFI_SSL)
             if not username or not passwd:
                 raise Exception("Error getting ONE credentials using TTS.")
             auth["username"] = username
@@ -329,6 +386,13 @@ class OpenNebulaCloudConnector(CloudConnector):
                 res_state = VirtualMachine.UNKNOWN
             vm.state = res_state
 
+            if vm.state == VirtualMachine.FAILED:
+                # in case of error try to get the error message
+                if res_vm.USER_TEMPLATE and res_vm.USER_TEMPLATE.ERROR:
+                    error_msg = res_vm.USER_TEMPLATE.ERROR
+                    if error_msg not in self.error_messages:
+                        self.error_messages += error_msg
+
             # Update network data
             self.setIPsFromTemplate(vm, res_vm.TEMPLATE)
 
@@ -554,23 +618,24 @@ class OpenNebulaCloudConnector(CloudConnector):
         cpu = system.getValue('cpu.count')
         arch = system.getValue('cpu.arch')
         memory = system.getFeature('memory.size').getValue('M')
-        name = system.getValue("instance_name")
-        if not name:
-            name = system.getValue("disk.0.image.name")
-        if not name:
-            name = "userimage"
+        name = self.gen_instance_name(system, False)
         url = urlparse(system.getValue("disk.0.image.url"))
         path = os.path.basename(url[2])
 
         if path.isdigit():
-            disks = 'DISK = [ IMAGE_ID = "%s" ]\n' % path
+            disks = 'DISK = [ IMAGE_ID = "%s" ' % path
         else:
             if ConfigOpenNebula.IMAGE_UNAME:
                 # This only works if the user owns the image
-                disks = 'DISK = [ IMAGE = "%s" ]\n' % path
+                disks = 'DISK = [ IMAGE = "%s" ' % path
             else:
-                disks = 'DISK = [ IMAGE = "%s", IMAGE_UNAME = "%s" ]\n' % (
+                disks = 'DISK = [ IMAGE = "%s", IMAGE_UNAME = "%s" ' % (
                     path, ConfigOpenNebula.IMAGE_UNAME)
+
+        # if system.getValue("disk.0.size"):
+        #     disks += ', SIZE=%d ' % system.getFeature("disk.0.size").getValue('M')
+        disks += ']\n'
+
         cont = 1
         while system.getValue("disk." + str(cont) + ".image.url") or system.getValue("disk." + str(cont) + ".size"):
             disk_image = system.getValue("disk." + str(cont) + ".image.url")
@@ -612,8 +677,9 @@ class OpenNebulaCloudConnector(CloudConnector):
         ''' % (name, cpu, cpu, memory, arch, disks, sched, ConfigOpenNebula.TEMPLATE_OTHER)
 
         user_template = ""
-        tags = self.get_instance_tags(system)
+        tags = self.get_instance_tags(system, auth_data, vm.inf)
         for key, value in tags.items():
+            key = key.replace("-", "_")
             user_template += '%s = "%s", ' % (key, value)
 
         if user_template:
@@ -831,7 +897,7 @@ class OpenNebulaCloudConnector(CloudConnector):
         return res
 
     @staticmethod
-    def map_radl_one_networks(radl_nets, one_nets):
+    def map_radl_one_networks(radl, one_nets):
         """
         Generate a mapping between the RADL networks and the ONE networks
 
@@ -844,50 +910,43 @@ class OpenNebulaCloudConnector(CloudConnector):
         """
         res = {}
 
+        system = radl.systems[0]
         used_nets = []
-        last_net = None
-        for radl_net in radl_nets:
-            # First check if the user has specified a provider ID
-            net_provider_id = radl_net.getValue('provider_id')
-            if net_provider_id:
-                for (net_name, net_id, is_public) in one_nets:
-                    # If the name is the same and have the same "publicity" value
-                    if ((net_id == net_provider_id or net_name == net_provider_id) and
-                            radl_net.isPublic() == is_public):
-                        res[radl_net.id] = (net_name, net_id, is_public)
-                        used_nets.append(net_id)
-                        break
-            else:
-                for (net_name, net_id, is_public) in one_nets:
-                    if net_id not in used_nets and radl_net.isPublic() == is_public:
-                        res[radl_net.id] = (net_name, net_id, is_public)
-                        used_nets.append(net_id)
-                        last_net = (net_name, net_id, is_public)
-                        break
-                if radl_net.id not in res:
-                    res[radl_net.id] = last_net
-
-        # In case of there are no private network, use public ones for non
-        # mapped networks
-        used_nets = []
-        for radl_net in radl_nets:
-            if radl_net.id not in res or not res[radl_net.id]:
+        for radl_net in radl.networks:
+            if system.getNumNetworkWithConnection(radl_net.id) is not None:
+                # First check if the user has specified a provider ID
                 net_provider_id = radl_net.getValue('provider_id')
                 if net_provider_id:
                     for (net_name, net_id, is_public) in one_nets:
-                        if net_name == net_provider_id:
+                        # If the name is the same and have the same "publicity" value
+                        if ((net_id == net_provider_id or net_name == net_provider_id) and
+                                radl_net.isPublic() == is_public):
                             res[radl_net.id] = (net_name, net_id, is_public)
                             used_nets.append(net_id)
                             break
                 else:
                     for (net_name, net_id, is_public) in one_nets:
-                        if net_id not in used_nets and is_public:
+                        if net_id not in used_nets and radl_net.isPublic() == is_public:
                             res[radl_net.id] = (net_name, net_id, is_public)
                             used_nets.append(net_id)
-                            last_net = (net_name, net_id, is_public)
                             break
-                    if radl_net.id not in res:
-                        res[radl_net.id] = last_net
+
+        # In case of there are no private network, use public ones for non
+        # mapped networks
+        for radl_net in radl.networks:
+            if system.getNumNetworkWithConnection(radl_net.id) is not None:
+                if radl_net.id not in res or not res[radl_net.id]:
+                    net_provider_id = radl_net.getValue('provider_id')
+                    if net_provider_id:
+                        for (net_name, net_id, is_public) in one_nets:
+                            if net_name == net_provider_id:
+                                res[radl_net.id] = (net_name, net_id, is_public)
+                                break
+                    else:
+                        for (net_name, net_id, is_public) in one_nets:
+                            if is_public:
+                                res[radl_net.id] = (net_name, net_id, is_public)
+                                break
 
         return res
 
@@ -909,7 +968,7 @@ class OpenNebulaCloudConnector(CloudConnector):
         if not one_nets:
             self.log_error("No ONE network found")
             return res
-        nets = self.map_radl_one_networks(radl.networks, one_nets)
+        nets = self.map_radl_one_networks(radl, one_nets)
 
         system = radl.systems[0]
         # First set the public ones (onecloud issues...)
@@ -1210,3 +1269,79 @@ class OpenNebulaCloudConnector(CloudConnector):
                     return image.ID
 
             return None
+
+    def list_images(self, auth_data, filters=None):
+        server = ServerProxy(self.server_url, allow_none=True)
+        session_id = self.getSessionID(auth_data)
+        if session_id is None:
+            return [(False, "Incorrect auth data, username and password must be specified for OpenNebula provider.")]
+
+        success, res_info = server.one.imagepool.info(session_id, -2, -1, -1)[0:2]
+
+        if success:
+            pool_info = IMAGE_POOL(res_info)
+        else:
+            self.logger.error("Error in the function one.imagepool.info: " + res_info)
+            raise Exception(res_info)
+
+        images = []
+        for image in pool_info.IMAGE:
+            if image.STATE in [IMAGE.STATE_READY, IMAGE.STATE_USED]:
+                images.append({"uri": "one://%s/%s" % (self.cloud.server, image.ID), "name": image.NAME})
+
+        return images
+
+    @staticmethod
+    def _get_public_ip_quota(net_quotas, net_info):
+        res = {"used": 0, "limit": None}
+
+        net_dict = {}
+        for _, net_id, public in net_info:
+            net_dict[net_id] = public
+
+        for net in net_quotas.NETWORK:
+            if net.ID in net_dict and net_dict[net.ID]:
+                res['used'] += net.LEASES_USED
+                # if one net has not limit, all has no limit
+                if res['limit'] != -1:
+                    if res['limit'] is None:
+                        res['limit'] = net.LEASES
+                    else:
+                        res['limit'] += net.LEASES
+
+        return res
+
+    def get_quotas(self, auth_data):
+        server = ServerProxy(self.server_url, allow_none=True)
+        session_id = self.getSessionID(auth_data)
+        if session_id is None:
+            return [(False, "Incorrect auth data, username and password must be specified for OpenNebula provider.")]
+
+        success, res_info = server.one.user.info(session_id, -1)[0:2]
+
+        if success:
+            user_info = USER(res_info)
+        else:
+            self.logger.error("Error in the function one.user.info: " + res_info)
+            raise Exception(res_info)
+
+        res = {"cores": {"used": 0, "limit": -1},
+               "ram": {"used": 0, "limit": -1},
+               "instances": {"used": 0, "limit": -1},
+               "security_groups": {"used": 0, "limit": -1}}
+
+        res["cores"]["used"] = user_info.VM_QUOTA.VM.CPU_USED
+        if user_info.VM_QUOTA.VM.CPU >= 0:
+            res["cores"]["limit"] = user_info.VM_QUOTA.VM.CPU
+        res["ram"]["used"] = user_info.VM_QUOTA.VM.MEMORY_USED / 1024
+        if user_info.VM_QUOTA.VM.MEMORY >= 0:
+            res["ram"]["limit"] = user_info.VM_QUOTA.VM.MEMORY / 1024
+        res["instances"]["used"] = user_info.VM_QUOTA.VM.VMS_USED
+        if user_info.VM_QUOTA.VM.VMS >= 0:
+            res["instances"]["limit"] = user_info.VM_QUOTA.VM.VMS
+
+        # Ine ONE map floating IPs to public IP leases
+        one_nets = self.getONENetworks(auth_data)
+        res["floating_ips"] = self._get_public_ip_quota(user_info.NETWORK_QUOTA, one_nets)
+
+        return res
