@@ -19,9 +19,9 @@ import time
 from netaddr import IPNetwork, IPAddress
 import os.path
 import tempfile
-from libcloud.common.exceptions import BaseHTTPError
 
 try:
+    from libcloud.common.exceptions import BaseHTTPError
     from libcloud.compute.types import Provider
     from libcloud.compute.providers import get_driver
     from libcloud.compute.base import NodeAuthSSHKey
@@ -55,6 +55,8 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
     """ Max number of retries to get a public IP """
     CONFIG_DRIVE = False
     """ Enable config drive """
+    CONFIRM_TIMEOUT = 120
+    """ Confirm Timeout """
 
     def __init__(self, cloud_info, inf):
         self.auth = None
@@ -184,29 +186,87 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
                     driver.image_connection.service_region = None
                     driver.network_connection.service_region = None
                     driver.volumev2_connection.service_region = None
+                    # To avoid error with old versions of LibCloud
+                    if 'volumev3_connection' in driver.__dict__:
+                        driver.volumev3_connection.service_region = None
 
             self.driver = driver
             return driver
 
-    def get_instance_type(self, sizes, radl):
-        """
-        Get the name of the instance type to launch to LibCloud
+    @staticmethod
+    def guess_instance_type_gpu(size, num_gpus, vendor=None, model=None):
+        """Try to guess if this NodeSize has GPU support"""
+        for k, v in size.extra.items():
+            if k.lower().find("gpu") != -1 and v.lower() not in ['false', 'no', '0']:
+                return True
+        # From EGI FedCloud TF
+        if 'Accelerator:Type' in size.extra and size.extra['Accelerator:Type'] == 'GPU':
+            if 'Accelerator:Number' in size.extra and size.extra['Accelerator:Number']:
+                if float(size.extra['Accelerator:Number']) < num_gpus:
+                    return False
+            else:
+                return False
+            if vendor:
+                if 'Accelerator:Vendor' in size.extra and size.extra['Accelerator:Vendor']:
+                    if size.extra['Accelerator:Vendor'].lower() != vendor.lower():
+                        return False
+                else:
+                    return False
+            if model:
+                if 'Accelerator:Model' in size.extra and size.extra['Accelerator:Model']:
+                    if size.extra['Accelerator:Model'].lower() != model.lower():
+                        return False
+                else:
+                    return False
+            return True
+        else:
+            return False
 
-        Arguments:
-           - size(list of :py:class: `libcloud.compute.base.NodeSize`): List of sizes on a provider
-           - radl(str): RADL document with the requirements of the VM to get the instance type
-        Returns: a :py:class:`libcloud.compute.base.NodeSize` with the instance type to launch
-        """
+    def get_list_sizes_details(self, driver):
+        """Assure to get extra_specs in all the sizes"""
+        res = []
+        for size in driver.list_sizes():
+            # If extra specs are not fetch in the list_sizes function
+            if len(size.extra) == 0:
+                try:
+                    # get it now
+                    size.extra = driver.ex_get_size_extra_specs(size.id)
+                except Exception:
+                    self.log_exception("Error trying to get flavor '%s' extra_specs." % size.id)
+
+            size.extra['pci_devices'] = 0
+            # Count to pci devices to enable sorting
+            if 'pci_passthrough:alias' in size.extra:
+                pci_devices = size.extra['pci_passthrough:alias'].split(',')
+                for device in pci_devices:
+                    device_info = device.split(':')
+                    if len(device_info) > 1:
+                        if device_info[1] and device_info[1].isnumeric():
+                            size.extra['pci_devices'] += int(device_info[1])
+                    else:
+                        size.extra['pci_devices'] += 1
+
+            res.append(size)
+
+        return res
+
+    def get_instance_type(self, driver, radl, location=None):
+        sizes = self.get_list_sizes_details(driver)
         instance_type_name = radl.getValue('instance_type')
 
         (cpu, cpu_op, memory, memory_op, disk_free, disk_free_op) = self.get_instance_selectors(radl, disk_unit="G")
+        num_gpus = radl.getValue('gpu.count')
+        gpu_model = radl.getValue('gpu.model')
+        gpu_vendor = radl.getValue('gpu.vendor')
 
         # get the node size with the lowest price, vcpus, memory and disk
-        sizes.sort(key=lambda x: (x.price, x.vcpus, x.ram, x.disk))
+        sizes.sort(key=lambda x: (x.price, x.vcpus, x.ram, x.disk, x.extra['pci_devices']))
         for size in sizes:
             comparison = cpu_op(size.vcpus, cpu)
             comparison = comparison and memory_op(size.ram, memory)
             comparison = comparison and disk_free_op(size.disk, disk_free)
+            if num_gpus and not self.guess_instance_type_gpu(size, num_gpus, gpu_vendor, gpu_model):
+                continue
 
             if comparison:
                 if not instance_type_name or size.name == instance_type_name:
@@ -234,7 +294,7 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
             driver = self.get_driver(auth_data)
 
             res_system = radl_system.clone()
-            instance_type = self.get_instance_type(driver.list_sizes(), res_system)
+            instance_type = self.get_instance_type(driver, res_system)
             self.update_system_info_from_instance(res_system, instance_type)
 
             username = res_system.getValue('disk.0.os.credentials.username')
@@ -336,7 +396,9 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
                     vol_id = vol_info['id']
                     self.log_debug("Getting Volume info %s" % vol_id)
                     volume = node.driver.ex_get_volume(vol_id)
-                    disk_size = vm.info.systems[0].getFeature("disk." + str(cont) + ".size").getValue('G')
+                    disk_size = None
+                    if vm.info.systems[0].getValue("disk." + str(cont) + ".size"):
+                        disk_size = vm.info.systems[0].getFeature("disk." + str(cont) + ".size").getValue('G')
                     if disk_size and disk_size != volume.size:
                         self.log_warn("Volume ID %s does not have the expected size %s != %s" % (vol_id,
                                                                                                  volume.size,
@@ -362,9 +424,21 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
         if node:
             vm.state = self.VM_STATE_MAP.get(node.state, VirtualMachine.UNKNOWN)
 
+            if vm.state == VirtualMachine.FAILED:
+                if 'fault' in node.extra and node.extra['fault']:
+                    error_msg = str(node.extra['fault']['message'])
+                    if error_msg not in self.error_messages:
+                        self.error_messages += error_msg
+
             try:
                 flavorId = node.extra['flavorId']
                 instance_type = node.driver.ex_get_size(flavorId)
+                if len(instance_type.extra) == 0:
+                    try:
+                        # get it now
+                        instance_type.extra = node.driver.ex_get_size_extra_specs(instance_type.id)
+                    except Exception:
+                        self.log_exception("Error trying to get flavor '%s' extra_specs." % instance_type.id)
                 self.update_system_info_from_instance(vm.info.systems[0], instance_type)
             except Exception as ex:
                 self.log_warn("Error updating VM info from flavor ID: %s" % get_ex_error(ex))
@@ -396,11 +470,14 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
                 for radl_net in vm.info.networks:
                     net_provider_id = radl_net.getValue('provider_id')
                     net_cidr = radl_net.getValue('cidr')
+                    if radl_net.id not in res:
+                        res[radl_net.id] = []
 
                     if net_provider_id:
                         if net_name == net_provider_id:
                             if radl_net.isPublic() == is_public:
-                                res[radl_net.id] = ip
+                                if ip not in res[radl_net.id]:
+                                    res[radl_net.id].append(ip)
                                 if ip in res["#UNMAPPED#"]:
                                     res["#UNMAPPED#"].remove(ip)
                                 break
@@ -408,19 +485,26 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
                                 # the ip not matches the is_public value
                                 if ip not in res["#UNMAPPED#"]:
                                     res["#UNMAPPED#"].append(ip)
+                        else:
+                            # the ip not matches the is_public value
+                            if ip not in res["#UNMAPPED#"]:
+                                res["#UNMAPPED#"].append(ip)
                     elif net_cidr and "*" in net_cidr:
                         # in this case the net is not connected to this VM
                         continue
                     elif net_cidr and IPAddress(ip) in IPNetwork(net_cidr):
-                        res[radl_net.id] = ip
+                        if ip not in res[radl_net.id]:
+                            res[radl_net.id].append(ip)
                         radl_net.setValue('provider_id', net_name)
                         if ip in res["#UNMAPPED#"]:
                             res["#UNMAPPED#"].remove(ip)
                         break
                     else:
-                        if radl_net.id not in res:
-                            if radl_net.isPublic() == is_public and vm.getNumNetworkWithConnection(radl_net.id):
-                                res[radl_net.id] = ip
+                        if not res[radl_net.id]:
+                            if (radl_net.isPublic() == is_public and
+                                    vm.getNumNetworkWithConnection(radl_net.id) is not None):
+                                if ip not in res[radl_net.id]:
+                                    res[radl_net.id].append(ip)
                                 radl_net.setValue('provider_id', net_name)
                                 if ip in res["#UNMAPPED#"]:
                                     res["#UNMAPPED#"].remove(ip)
@@ -434,7 +518,10 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
                 added = False
                 for radl_net in vm.info.networks:
                     if radl_net.id not in res and radl_net.isPublic() == is_public:
-                        res[radl_net.id] = ip
+                        if radl_net.id not in res:
+                            res[radl_net.id] = []
+                        if ip not in res[radl_net.id]:
+                            res[radl_net.id].append(ip)
                         added = True
                         break
 
@@ -479,7 +566,7 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
                     ip = ipo['addr']
                     is_private = any([IPAddress(ip) in IPNetwork(mask) for mask in Config.PRIVATE_NET_MASKS])
 
-                    if ipo['OS-EXT-IPS:type'] == 'floating':
+                    if 'OS-EXT-IPS:type' in ipo and ipo['OS-EXT-IPS:type'] == 'floating':
                         ip_net_map[ip] = (None, not is_private)
                     else:
                         ip_net_map[ip] = (net_name, not is_private)
@@ -503,12 +590,12 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
                     system.delValue('net_interface.%d.ip' % i)
                 net_name = system.getValue("net_interface." + str(i) + ".connection")
                 if net_name in map_nets:
-                    ip = map_nets[net_name]
-                    if IPAddress(ip).version == 6:
-                        system.setValue("net_interface." + str(i) + ".ipv6", ip)
-                    else:
-                        system.setValue("net_interface." + str(i) + ".ip", ip)
-                    ips_assigned.append(ip)
+                    for ip in map_nets[net_name]:
+                        if IPAddress(ip).version == 6:
+                            system.setValue("net_interface." + str(i) + ".ipv6", ip)
+                        else:
+                            system.setValue("net_interface." + str(i) + ".ip", ip)
+                        ips_assigned.append(ip)
                 i += 1
 
             # For IPs not correctly mapped
@@ -516,13 +603,14 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
             # not correctly mapped
             for net_name, ip in map_nets.items():
                 if net_name != '#UNMAPPED#':
-                    if ip not in ips_assigned:
-                        num_net = system.getNumNetworkIfaces()
-                        if IPAddress(ip).version == 6:
-                            system.setValue('net_interface.' + str(num_net) + '.ipv6', ip)
-                        else:
-                            system.setValue('net_interface.' + str(num_net) + '.ip', ip)
-                        system.setValue('net_interface.' + str(num_net) + '.connection', net_name)
+                    for ipu in ip:
+                        if ipu not in ips_assigned:
+                            num_net = system.getNumNetworkIfaces()
+                            if IPAddress(ipu).version == 6:
+                                system.setValue('net_interface.' + str(num_net) + '.ipv6', ipu)
+                            else:
+                                system.setValue('net_interface.' + str(num_net) + '.ip', ipu)
+                            system.setValue('net_interface.' + str(num_net) + '.connection', net_name)
                 else:
                     pub_ips = []
                     priv_ips = []
@@ -560,7 +648,21 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
         if instance_type:
             LibCloudCloudConnector.update_system_info_from_instance(system, instance_type)
             if instance_type.vcpus:
-                system.addFeature(Feature("cpu.count", "=", instance_type.vcpus), conflict="me", missing="other")
+                system.addFeature(Feature("cpu.count", "=", instance_type.vcpus), conflict="other", missing="other")
+            if instance_type.ephemeral_disk:
+                system.addFeature(Feature("disk.0.free_size", "=", instance_type.ephemeral_disk),
+                                  conflict="other", missing="other")
+            # From EGI FedCloud TF
+            if 'Accelerator:Type' in instance_type.extra and instance_type.extra['Accelerator:Type'] == 'GPU':
+                if 'Accelerator:Number' in instance_type.extra:
+                    system.addFeature(Feature("gpu.count", "=", float(instance_type.extra['Accelerator:Number'])),
+                                      conflict="other", missing="other")
+                if 'Accelerator:Vendor' in instance_type.extra:
+                    system.addFeature(Feature("gpu.vendor", "=", instance_type.extra['Accelerator:Vendor']),
+                                      conflict="other", missing="other")
+                if 'Accelerator:Model' in instance_type.extra:
+                    system.addFeature(Feature("gpu.model", "=", instance_type.extra['Accelerator:Model']),
+                                      conflict="other", missing="other")
 
     @staticmethod
     def get_ost_net(driver, name=None, netid=None):
@@ -892,11 +994,19 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
 
         boot_disk = {
             'boot_index': 0,
-            'device_name': 'vda',
             'source_type': "image",
             'delete_on_termination': False,
             'uuid': image.id
         }
+
+        # if the user sets the size, we create a volume with this size
+        # instead of booting directly from the image
+        if system.getValue("disk.0.size"):
+            boot_disk['destination_type'] = 'volume'
+            boot_disk['volume_size'] = system.getFeature("disk.0.size").getValue('G')
+            boot_disk['delete_on_termination'] = True
+            del boot_disk['device_name']
+
         res = [boot_disk]
 
         cont = 1
@@ -941,6 +1051,15 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
 
         return res
 
+    def get_vo_name(self, auth_data):
+        """Get vo field from auth data if present"""
+        auths = auth_data.getAuthInfo(self.type, self.cloud.server)
+        if auths:
+            auth = auths[0]
+            if 'vo' in auth and auth['vo']:
+                return auth['vo']
+        return None
+
     def launch(self, inf, radl, requested_radl, num_vm, auth_data):
         driver = self.get_driver(auth_data)
 
@@ -948,7 +1067,8 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
 
         image_url = system.getValue("disk.0.image.url")
         if urlparse(image_url)[0] == "appdb":
-            _, image_id, msg = AppDB.get_image_data(image_url, "openstack")
+            vo = self.get_vo_name(auth_data)
+            _, image_id, msg = AppDB.get_image_data(image_url, "openstack", vo)
             if not image_id:
                 self.log_error(msg)
                 raise Exception("Error in appdb image: %s" % msg)
@@ -956,16 +1076,10 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
             image_id = self.get_image_id(system.getValue("disk.0.image.url"))
         image = driver.get_image(image_id)
 
-        instance_type = self.get_instance_type(driver.list_sizes(), system)
+        instance_type = self.get_instance_type(driver, system)
         if not instance_type:
             raise Exception("No flavor found for the specified VM requirements.")
 
-        name = system.getValue("instance_name")
-        if not name:
-            name = system.getValue("disk.0.image.name")
-        if not name:
-            name = "userimage"
-
         blockdevicemappings = self.get_volumes(driver, image, radl)
 
         with inf._lock:
@@ -979,10 +1093,12 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
                 'networks': nets,
                 'image': image,
                 'ex_security_groups': sgs,
-                'ex_blockdevicemappings': blockdevicemappings,
-                'name': "%s-%s" % (name, int(time.time() * 100))}
+                'name': self.gen_instance_name(system)}
 
-        tags = self.get_instance_tags(system)
+        if blockdevicemappings:
+            args['ex_blockdevicemappings'] = blockdevicemappings
+
+        tags = self.get_instance_tags(system, auth_data, inf)
         if tags:
             args['ex_metadata'] = tags
 
@@ -1046,7 +1162,10 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
         if all_failed:
             for sg in sgs:
                 self.log_info("Deleting security group: %s." % sg.id)
-                driver.ex_delete_security_group(sg)
+                try:
+                    driver.ex_delete_security_group(sg)
+                except Exception as ex:
+                    self.log_exception("Error deleting security group: %s." % get_ex_error(ex))
 
         return res
 
@@ -1313,13 +1432,13 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
                 except Exception as ex:
                     self.log_exception("Error dettaching volume %s." % vol_id)
 
-            try:
-                for sg_name in self._get_security_names(vm.inf):
+            for sg_name in self._get_security_names(vm.inf):
+                try:
                     self.log_debug("Dettaching SG %s." % sg_name)
                     security_group = OpenStackSecurityGroup(None, None, sg_name, "", node.driver)
                     node.driver.ex_remove_security_group_from_node(security_group, node)
-            except Exception as ex:
-                self.log_exception("Error dettaching SG %s." % sg_name)
+                except Exception as ex:
+                    self.log_exception("Error dettaching SG %s." % sg_name)
 
             res = node.destroy()
             success.append(res)
@@ -1517,7 +1636,7 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
                 self.log_debug("No memory nor cpu nor instance_type specified. VM not resized.")
                 return (True, "")
             else:
-                instance_type = self.get_instance_type(node.driver.list_sizes(), radl.systems[0])
+                instance_type = self.get_instance_type(node.driver, radl.systems[0])
                 if instance_type is None:
                     return (False, "Error resizing VM: No instance type found.")
                 if node.extra['flavorId'] != instance_type.id:
@@ -1527,13 +1646,16 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
                         if success:
                             cont = 0
                             # wait the node to be in correct state to confirm
-                            while node.extra['vm_state'] != 'resized' and cont < 30:
-                                time.sleep(2)
-                                cont += 2
+                            while node.extra['vm_state'] != 'resized' and cont < self.CONFIRM_TIMEOUT:
+                                time.sleep(3)
+                                cont += 3
                                 self.log_debug("Confirming resize of the node: %s" % node.id)
                                 node = self.get_node_with_id(vm.id, auth_data)
 
-                            success = node.driver.ex_confirm_resize(node)
+                            if node.extra['vm_state'] == 'resized':
+                                success = node.driver.ex_confirm_resize(node)
+                            else:
+                                return (False, "Error resizing VM: Resize cannot be confirmed.")
                     except Exception as ex:
                         self.log_exception("Error resizing VM.")
                         return (False, "Error resizing VM: " + str(ex))
@@ -1617,7 +1739,7 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
                 return (True, "")
             except Exception as ex:
                 self.log_exception("Error connecting with OpenStack server")
-                return (False, "Error connecting with OpenStack server: " + str(ex))
+                return (False, "Error connecting with OpenStack server: " + get_ex_error(ex))
 
     def alter_public_ips(self, vm, radl, auth_data):
         """
@@ -1655,7 +1777,7 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
                     return False, "Error detaching IP %s from node %s" % (current_public_ip, node.id)
         except Exception as ex:
             self.log_exception("Error adding/removing new public IP")
-            return (False, "Error adding/removing new public IP: " + str(ex))
+            return (False, "Error adding/removing new public IP: " + get_ex_error(ex))
         return True, ""
 
     def delete_elastic_ips(self, node, vm):
@@ -1690,3 +1812,55 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
         except Exception as ex:
             self.log_exception("Error removing Floating IPs to VM ID: " + str(vm.id))
             return False, "Error removing Floating IPs: %s" % get_ex_error(ex)
+
+    def list_images(self, auth_data, filters=None):
+        driver = self.get_driver(auth_data)
+        images = []
+        for image in driver.list_images():
+            if 'status' not in image.extra or image.extra['status'] == 'active':
+                images.append({"uri": "ost://%s/%s" % (self.cloud.server, image.id), "name": image.name})
+        return images
+
+    @staticmethod
+    def _get_tenant_id(driver, auth):
+        """
+        Workaround function to get tenant id from tenant name
+        """
+        if 'auth_version' in auth and auth['auth_version'] == '3.x_oidc_access_token':
+            return auth['domain']
+        else:
+            if 'tenant_id' in auth:
+                return auth['tenant_id']
+            else:
+                return auth['tenant']
+
+        return None
+
+    def get_quotas(self, auth_data):
+        driver = self.get_driver(auth_data)
+        tenant_id = self._get_tenant_id(driver, auth_data.getAuthInfo(self.type, self.cloud.server)[0])
+        quotas = driver.ex_get_quota_set(tenant_id)
+        try:
+            net_quotas = driver.ex_get_network_quotas(tenant_id)
+        except Exception:
+            net_quotas = None
+
+        quotas_dict = {}
+        quotas_dict["cores"] = {"used": quotas.cores.in_use + quotas.cores.reserved,
+                                "limit": quotas.cores.limit}
+        quotas_dict["ram"] = {"used": (quotas.ram.in_use + quotas.ram.reserved) / 1024,
+                              "limit": quotas.ram.limit / 1024}
+        quotas_dict["instances"] = {"used": quotas.instances.in_use + quotas.instances.reserved,
+                                    "limit": quotas.instances.limit}
+        quotas_dict["floating_ips"] = {"used": quotas.floating_ips.in_use + quotas.floating_ips.reserved,
+                                       "limit": quotas.floating_ips.limit}
+        quotas_dict["security_groups"] = {"used": quotas.security_groups.in_use + quotas.security_groups.reserved,
+                                          "limit": quotas.security_groups.limit}
+
+        if net_quotas:
+            quotas_dict["floating_ips"] = {"used": net_quotas.floatingip.in_use + net_quotas.floatingip.reserved,
+                                           "limit": net_quotas.floatingip.limit}
+            quotas_dict["security_groups"] = {"used": net_quotas.security_group.in_use +
+                                              net_quotas.security_group.reserved,
+                                              "limit": net_quotas.security_group.limit}
+        return quotas_dict
