@@ -4,6 +4,7 @@ import yaml
 import copy
 import operator
 import requests
+from toscaparser.nodetemplate import NodeTemplate
 
 try:
     unicode("hola")
@@ -18,7 +19,8 @@ from toscaparser.tosca_template import ToscaTemplate
 from toscaparser.elements.interfaces import InterfacesDef
 from toscaparser.functions import Function, is_function, get_function, GetAttribute, Concat, Token
 from IM.ansible_utils import merge_recipes
-from radl.radl import system, deploy, network, Feature, Features, configure, contextualize_item, RADL, contextualize
+from radl.radl import (system, deploy, network, Feature, Features, configure,
+                       contextualize_item, RADL, contextualize, ansible)
 
 
 class Tosca:
@@ -52,7 +54,7 @@ class Tosca:
         Get the specified property of the deployment based on policies
         """
         for policy in self.tosca.policies:
-            if policy.type_definition.type == "tosca.policies.Placement":
+            if policy.type_definition.type in ["tosca.policies.Placement", "tosca.policies.indigo.Placement"]:
                 node_list = []
                 if policy.targets_type == "node_templates":
                     node_list = policy.targets_list
@@ -90,7 +92,7 @@ class Tosca:
 
         radl = RADL()
         interfaces = {}
-        cont_intems = []
+        cont_items = []
 
         # first process the networks as they are referred later
         for node in self.tosca.nodetemplates:
@@ -105,6 +107,32 @@ class Tosca:
             if root_type in ["tosca.nodes.BlockStorage", "tosca.nodes.network.Port", "tosca.nodes.network.Network"]:
                 # These elements are processed in other parts
                 pass
+            elif root_type == "tosca.nodes.im.AnsibleHost":
+                # Only allow 1 ansible_host per document
+                ansible_host = ansible("ansible_host", None)
+                node_props = node.get_properties()
+                if node_props and "host" in node_props:
+                    host = self._final_function_result(node_props["host"].value, node)
+                    if host:
+                        ansible_host.setValue("host", host)
+                if node_props and "credential" in node_props:
+                    credentials = self._final_function_result(node_props["credential"].value, node)
+                    if 'user' in credentials:
+                        ansible_host.setValue("credentials.username", credentials['user'])
+                    token_type = "password"
+                    if 'token_type' in credentials and credentials['token_type']:
+                        token_type = credentials['token_type']
+                    token = None
+                    if 'token' in credentials and credentials['token']:
+                        token = credentials['token']
+                    if token:
+                        if token_type == "password":
+                            ansible_host.setValue("credentials.password", token)
+                        elif token_type == "private_key":
+                            ansible_host.setValue("credentials.private_key", token)
+                        else:
+                            Tosca.logger.warn("Unknown tyoe of token %s. Ignoring." % token_type)
+                radl.ansible_hosts = [ansible_host]
             else:
                 if root_type == "tosca.nodes.Compute":
                     # Add the system RADL element
@@ -151,16 +179,15 @@ class Tosca:
                 interfaces = Tosca._get_interfaces(node)
                 interfaces.update(Tosca._get_relationships_interfaces(relationships, node))
 
-                conf = self._gen_configure_from_interfaces(node, interfaces)
+                conf = self._gen_configure_from_interfaces(node, compute, interfaces)
                 if conf:
                     level = Tosca._get_dependency_level(node)
                     radl.configures.append(conf)
                     if compute:
-                        cont_intems.append(contextualize_item(
-                            compute.name, conf.name, level))
+                        cont_items.append(contextualize_item(compute.name, conf.name, level))
 
-        if cont_intems:
-            radl.contextualize = contextualize(cont_intems)
+        if cont_items:
+            radl.contextualize = contextualize(cont_items)
         else:
             # If there are no configures, disable contextualization
             radl.contextualize = contextualize({})
@@ -284,6 +311,8 @@ class Tosca:
             else:
                 if "source" in port:
                     remote_port = port["source"]
+                else:
+                    raise Exception("source port must be specified in PortSpec type.")
                 if "target" in port:
                     local_port = port["target"]
                 else:
@@ -513,24 +542,52 @@ class Tosca:
         for rel_tpl in rel_tpls:
             if rel.type == rel_tpl.type:
                 return rel_tpl
+            else:
+                root_type = Tosca._get_root_parent_type(rel_tpl).type
+                if root_type == rel.type:
+                    return rel_tpl
+        return None
 
     @staticmethod
     def _get_relationships_interfaces(relationships, node):
         res = {}
         for src, trgt, rel in relationships:
             rel_tpl = Tosca._get_relationship_template(rel, src, trgt)
-            if rel_tpl.interfaces:
-                if src.name == node.name:
-                    for name in ['pre_configure_source', 'post_configure_source', 'add_source']:
+
+            rel_tlp_def_interfaces = {}
+            if rel_tpl.type_definition.interfaces and 'Standard' in rel_tpl.type_definition.interfaces:
+                rel_tlp_def_interfaces = rel_tpl.type_definition.interfaces['Standard']
+
+            if src.name == node.name:
+                # Also add the configure of the target node of the relation
+                trgt_interfaces = Tosca._get_interfaces(trgt, ['pre_configure_source', 'post_configure_source'])
+                for name in ['pre_configure_source', 'post_configure_source', 'add_source']:
+                    if trgt_interfaces and name in trgt_interfaces:
+                        res[name] = trgt_interfaces[name]
+                    if rel_tpl.interfaces:
                         for iface in rel_tpl.interfaces:
                             if iface.name == name:
                                 res[name] = iface
-                elif trgt.name == node.name:
-                    for name in ['pre_configure_target', 'post_configure_target', 'add_target',
-                                 'target_changed', 'remove_target']:
+                    if rel_tlp_def_interfaces and name in rel_tlp_def_interfaces:
+                        res[name] = InterfacesDef(rel_tpl.type_definition, 'Standard',
+                                                  name=name, value=rel_tlp_def_interfaces[name],
+                                                  node_template=rel_tpl)
+
+            elif trgt.name == node.name:
+                src_interfaces = Tosca._get_interfaces(src, ['pre_configure_target', 'post_configure_target'])
+                for name in ['pre_configure_target', 'post_configure_target', 'add_target',
+                             'target_changed', 'remove_target']:
+                    if src_interfaces and name in src_interfaces:
+                        res[name] = src_interfaces[name]
+                    if rel_tpl.interfaces:
                         for iface in rel_tpl.interfaces:
                             if iface.name == name:
                                 res[name] = iface
+                    if rel_tlp_def_interfaces and name in rel_tlp_def_interfaces:
+                        res[name] = InterfacesDef(rel_tpl.type_definition, 'Standard',
+                                                  name=name, value=rel_tlp_def_interfaces[name],
+                                                  node_template=rel_tpl)
+
         return res
 
     def _get_artifact_full_uri(self, node, artifact_name):
@@ -566,7 +623,7 @@ class Tosca:
 
         return res
 
-    def _gen_configure_from_interfaces(self, node, interfaces):
+    def _gen_configure_from_interfaces(self, node, compute, interfaces):
         if not interfaces:
             return None
 
@@ -575,11 +632,14 @@ class Tosca:
         recipe_list = []
         remote_artifacts_path = "/tmp"
         # Take the interfaces in correct order
-        for name in ['create', 'pre_configure_source', 'pre_configure_target', 'configure',
-                     'post_configure_source', 'post_configure_target', 'start', 'add_target',
-                     'add_source', 'target_changed', 'remove_target']:
+        for name in ['create', 'pre_configure_source', 'pre_configure_target', 'configure_rel',
+                     'configure', 'post_configure_source', 'post_configure_target', 'start',
+                     'add_target', 'add_source', 'target_changed', 'remove_target']:
             interface = interfaces.get(name, None)
             if interface:
+                if interface.node_template:
+                    orig_node = node
+                    node = interface.node_template
                 artifacts = []
                 # Get the inputs
                 env = {}
@@ -624,7 +684,10 @@ class Tosca:
                         raise Exception("Error downloading the implementation script '%s': %s" % (
                             implementation, str(ex)))
                 else:
-                    script_path = os.path.join(Tosca.ARTIFACTS_PATH, implementation)
+                    if implementation_url[0] == 'file':
+                        script_path = implementation_url[2]
+                    else:
+                        script_path = os.path.join(Tosca.ARTIFACTS_PATH, implementation)
                     if os.path.isfile(script_path):
                         f = open(script_path)
                         script_content = f.read()
@@ -672,7 +735,12 @@ class Tosca:
                     recipe_list.append(recipe)
 
         if tasks or recipe_list:
-            name = node.name + "_conf"
+            name = node.name
+            if node.name != compute.name:
+                name = name + "_" + compute.name
+            if node.name != orig_node.name:
+                name = name + "_" + orig_node.name
+            name = name + "_conf"
             if variables:
                 recipes = "---\n- vars:\n" + variables + "\n"
                 recipes += "  "
@@ -1013,31 +1081,35 @@ class Tosca:
         Take a translator.toscalib.functions.Function and return the final result
         (in some cases the result of a function is another function)
         """
-        if isinstance(func, dict):
-            if is_function(func):
-                func = get_function(self.tosca, node, func)
-
-        while isinstance(func, Function):
+        if not isinstance(func, (Function, dict, list)):
+            return func
+        elif isinstance(func, Function):
             if isinstance(func, GetAttribute):
                 func = self._get_attribute_result(func, node, inf_info)
             elif isinstance(func, Concat):
-                func = self._get_intrinsic_value(
-                    {"concat": func.args}, node, inf_info)
+                func = self._get_intrinsic_value({"concat": func.args}, node, inf_info)
             elif isinstance(func, Token):
-                func = self._get_intrinsic_value(
-                    {"token": func.args}, node, inf_info)
+                func = self._get_intrinsic_value({"token": func.args}, node, inf_info)
             else:
                 func = func.result()
-
-        if isinstance(func, dict):
-            if self._is_intrinsic(func):
+            return self._final_function_result(func, node, inf_info)
+        elif isinstance(func, list):
+            for i, elem in enumerate(func):
+                func[i] = self._final_function_result(elem, node, inf_info)
+            return func
+        else:  # is a dict
+            if is_function(func):
+                func = get_function(self.tosca, node, func)
+                return self._final_function_result(func, node, inf_info)
+            elif self._is_intrinsic(func):
                 func = self._get_intrinsic_value(func, node, inf_info)
-
-        if func is None:
-            # TODO: resolve function values related with run-time values as IM
-            # or ansible variables
-            pass
-        return func
+                return self._final_function_result(func, node, inf_info)
+            else:  # a plain dict
+                for k, v in func.items():
+                    func[k] = self._final_function_result(v, node, inf_info)
+                return func
+        # TODO: resolve function values related with run-time values as IM
+        # or ansible variables
 
     def _find_host_compute(self, node, nodetemplates):
         """
@@ -1245,15 +1317,24 @@ class Tosca:
     @staticmethod
     def _get_node_artifacts(node):
         """ Get a dict will the node artifacts """
-        artifacts_dict = {}
-        artifacts = node.type_definition.get_value('artifacts', node.entity_tpl, True)
+        artifacts = []
+        artifacts.append(node.type_definition.get_value('artifacts', node.entity_tpl, True))
 
-        if artifacts:
-            if isinstance(artifacts, dict):
-                artifacts_dict = artifacts
-            else:
-                for elem in artifacts:
-                    artifacts_dict.update(elem)
+        if (isinstance(node, NodeTemplate)):
+            # Get also artifacts of related nodes and relations
+            for relationship, trgt in node.relationships.items():
+                artifacts.append(relationship.get_value('artifacts'))
+                artifacts.append(trgt.type_definition.get_value('artifacts', trgt.entity_tpl, True))
+
+        artifacts_dict = {}
+
+        for artifact in artifacts:
+            if artifact:
+                if isinstance(artifact, dict):
+                    artifacts_dict.update(artifact)
+                else:
+                    for elem in artifact:
+                        artifacts_dict.update(elem)
 
         return artifacts_dict
 
@@ -1295,6 +1376,18 @@ class Tosca:
 
         res.setValue("instance_name", node.name)
 
+        for prop in node.get_properties_objects():
+            if prop.name == "tags" and prop.value:
+                if isinstance(prop.value, dict):
+                    instance_tags = ""
+                    for k, v in prop.value.items():
+                        if instance_tags != "":
+                            instance_tags += ","
+                        instance_tags += "%s=%s" % (k, v)
+                    res.setValue("instance_tags", instance_tags)
+                else:
+                    raise Exception("tags property must be a dictionary.")
+
         property_map = {
             'architecture': 'cpu.arch',
             'type': 'disk.0.os.name',
@@ -1303,11 +1396,14 @@ class Tosca:
             'image': 'disk.0.image.url',
             'credential': 'disk.0.os.credentials',
             'num_cpus': 'cpu.count',
-            'disk_size': 'disks.free_size',
+            'disk_size': 'disk.0.size',
             'mem_size': 'memory.size',
             'cpu_frequency': 'cpu.performance',
             'instance_type': 'instance_type',
             'preemtible_instance': 'spot',
+            'num_gpus': 'gpu.count',
+            'gpu_vendor': 'gpu.vendor',
+            'gpu_model': 'gpu.model',
         }
 
         for cap_type in ['os', 'host']:
@@ -1371,11 +1467,11 @@ class Tosca:
                     res.setValue('disk.%d.type' % num, vol_type)
                 if size:
                     res.setValue('disk.%d.size' % num, size, unit)
-                if device:
-                    res.setValue('disk.%d.device' % num, device)
-                if location:
-                    res.setValue('disk.%d.mount_path' % num, location)
-                    res.setValue('disk.%d.fstype' % num, fstype)
+            if device:
+                res.setValue('disk.%d.device' % num, device)
+            if location:
+                res.setValue('disk.%d.mount_path' % num, location)
+                res.setValue('disk.%d.fstype' % num, fstype)
 
         self._add_ansible_roles(node, nodetemplates, res)
 
@@ -1496,7 +1592,7 @@ class Tosca:
                 return node_type
 
     @staticmethod
-    def _get_interfaces(node):
+    def _get_interfaces(node, steps=['create', 'configure', 'start', 'stop', 'delete']):
         """
         Get a dict of InterfacesDef of the specified node
         """
@@ -1509,10 +1605,10 @@ class Tosca:
         while True:
             if node_type.interfaces and 'Standard' in node_type.interfaces:
                 for name, elems in node_type.interfaces['Standard'].items():
-                    if name in ['create', 'configure', 'start', 'stop', 'delete']:
+                    if name in steps:
                         if name not in interfaces:
-                            interfaces[name] = InterfacesDef(
-                                node_type, 'Standard', name=name, value=elems)
+                            interfaces[name] = InterfacesDef(node_type, 'Standard', name=name,
+                                                             value=elems, node_template=node)
 
             if node_type.parent_type is not None:
                 node_type = node_type.parent_type
