@@ -202,6 +202,18 @@ def CreateDiskSnapshot(inf_id, vm_id, disk_num, image_name, auto_delete, auth_da
     return WaitRequest(request)
 
 
+def GetCloudImageList(cloud_id, auth_data, filters=None):
+    request = IMBaseRequest.create_request(
+        IMBaseRequest.GET_CLOUD_IMAGE_LIST, (cloud_id, auth_data, filters))
+    return WaitRequest(request)
+
+
+def GetCloudQuotas(cloud_id, auth_data):
+    request = IMBaseRequest.create_request(
+        IMBaseRequest.GET_CLOUD_QUOTAS, (cloud_id, auth_data))
+    return WaitRequest(request)
+
+
 def launch_daemon():
     """
     Launch the IM daemon
@@ -245,6 +257,8 @@ def launch_daemon():
     server.register_function(GetInfrastructureState)
     server.register_function(GetVersion)
     server.register_function(CreateDiskSnapshot)
+    server.register_function(GetCloudImageList)
+    server.register_function(GetCloudQuotas)
 
     InfrastructureManager.logger.info(
         '************ Start Infrastructure Manager daemon (v.%s) ************' % version)
